@@ -1,5 +1,6 @@
 /* BFD COFF object file private structure.
-   Copyright (C) 1990, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1990, 91, 92, 93, 94, 95, 96, 97, 98, 1999
+   Free Software Foundation, Inc.
    Written by Cygnus Support.
 
 ** NOTE: libcoff.h is a GENERATED file.  Don't change it; instead,
@@ -55,7 +56,7 @@ typedef struct coff_tdata
   file_ptr sym_filepos;
 
   struct coff_ptr_struct *raw_syments;
-  unsigned int raw_syment_count;
+  unsigned long raw_syment_count;
 
   /* These are only valid once writing has begun */
   long int relocbase;
@@ -95,6 +96,14 @@ typedef struct coff_tdata
 
   /* Used by coff_find_nearest_line.  */
   PTR line_info;
+
+  /* The timestamp from the COFF file header.  */
+  long timestamp;
+
+  /* Copy of some of the f_flags bits in the COFF filehdr structure,
+     used by ARM code.  */
+  flagword flags;
+
 } coff_data_type;
 
 /* Tdata for pe image files. */
@@ -207,12 +216,14 @@ struct xcoff_section_tdata
 #define xcoff_section_data(abfd, sec) \
   ((struct xcoff_section_tdata *) coff_section_data ((abfd), (sec))->tdata)
 
-/* Tdata for sections in PEI image files.  */
+/* Tdata for sections in PE files.  */
 
 struct pei_section_tdata
 {
   /* The virtual size of the section.  */
   bfd_size_type virt_size;
+  /* The PE section flags.  */
+  long pe_flags;
 };
 
 /* An accessor macro for the pei_section_tdata structure.  */
@@ -243,6 +254,11 @@ struct coff_link_hash_entry
 
   /* Pointer to array of auxiliary entries, if any.  */
   union internal_auxent *aux;
+
+  /* Flag word; legal values follow.  */
+  unsigned short coff_link_hash_flags;
+  /* Symbol is a PE section symbol.  */
+#define COFF_LINK_HASH_PE_SECTION_SYMBOL (01)
 };
 
 /* COFF linker hash table.  */
@@ -296,6 +312,7 @@ extern void coff_print_symbol PARAMS ((bfd *, PTR filep, asymbol *,
 				       bfd_print_symbol_type how));
 extern void coff_get_symbol_info PARAMS ((bfd *, asymbol *,
 					  symbol_info *ret));
+extern boolean _bfd_coff_is_local_label_name PARAMS ((bfd *, const char *));
 extern asymbol *coff_bfd_make_debug_symbol PARAMS ((bfd *, PTR,
 						    unsigned long));
 extern boolean coff_find_nearest_line PARAMS ((bfd *,
@@ -415,6 +432,10 @@ struct coff_final_link_info
   bfd *output_bfd;
   /* Used to indicate failure in traversal routine.  */
   boolean failed;
+  /* If doing "task linking" set only during the time when we want the
+     global symbol writer to convert the storage class of defined global
+     symbols from global to static. */
+  boolean global_to_static;
   /* Hash table for long symbol names.  */
   struct bfd_strtab_hash *strtab;
   /* When doing a relocateable link, an array of information kept for
@@ -451,6 +472,41 @@ struct coff_final_link_info
   struct internal_reloc *internal_relocs;
 };
 
+/* Most COFF variants have no way to record the alignment of a
+   section.  This struct is used to set a specific alignment based on
+   the name of the section.  */
+
+struct coff_section_alignment_entry
+{
+  /* The section name.  */
+  const char *name;
+
+  /* This is either (unsigned int) -1, indicating that the section
+     name must match exactly, or it is the number of letters which
+     must match at the start of the name.  */
+  unsigned int comparison_length;
+
+  /* These macros may be used to fill in the first two fields in a
+     structure initialization.  */
+#define COFF_SECTION_NAME_EXACT_MATCH(name) (name), ((unsigned int) -1)
+#define COFF_SECTION_NAME_PARTIAL_MATCH(name) (name), (sizeof (name) - 1)
+
+  /* Only use this entry if the default section alignment for this
+     target is at least that much (as a power of two).  If this field
+     is COFF_ALIGNMENT_FIELD_EMPTY, it should be ignored.  */
+  unsigned int default_alignment_min;
+
+  /* Only use this entry if the default section alignment for this
+     target is no greater than this (as a power of two).  If this
+     field is COFF_ALIGNMENT_FIELD_EMPTY, it should be ignored.  */
+  unsigned int default_alignment_max;
+
+#define COFF_ALIGNMENT_FIELD_EMPTY ((unsigned int) -1)
+
+  /* The desired alignment for this section (as a power of two).  */
+  unsigned int alignment_power;
+};
+
 extern struct bfd_hash_entry *_bfd_coff_link_hash_newfunc
   PARAMS ((struct bfd_hash_entry *, struct bfd_hash_table *, const char *));
 extern boolean _bfd_coff_link_hash_table_init
@@ -477,6 +533,8 @@ extern struct bfd_hash_entry *_bfd_coff_debug_merge_hash_newfunc
   PARAMS ((struct bfd_hash_entry *, struct bfd_hash_table *, const char *));
 extern boolean _bfd_coff_write_global_sym
   PARAMS ((struct coff_link_hash_entry *, PTR));
+extern boolean _bfd_coff_write_task_globals
+  PARAMS ((struct coff_link_hash_entry *, PTR));
 extern boolean _bfd_coff_link_input_bfd
   PARAMS ((struct coff_final_link_info *, bfd *));
 extern boolean _bfd_coff_reloc_link_order
@@ -505,6 +563,13 @@ extern boolean _bfd_ppc_xcoff_relocate_section
   PARAMS ((bfd *, struct bfd_link_info *, bfd *, asection *, bfd_byte *,
 	   struct internal_reloc *, struct internal_syment *, asection **));
 
+/* Functions in coff-ppc.c.  FIXME: These are called be pe.em in the
+   linker, and so should start with bfd and be declared in bfd.h.  */
+
+extern boolean ppc_allocate_toc_section PARAMS ((struct bfd_link_info *));
+extern boolean ppc_process_before_allocation
+  PARAMS ((bfd *, struct bfd_link_info *));
+
 /* And more taken from the source .. */
 
 typedef struct coff_ptr_struct
@@ -561,6 +626,22 @@ struct lineno_cache_entry *lineno;
     /* Have the line numbers been relocated yet ? */
 boolean done_lineno;
 } coff_symbol_type;
+ /* COFF symbol classifications.  */
+
+enum coff_symbol_classification
+{
+   /* Global symbol.  */
+  COFF_SYMBOL_GLOBAL,
+   /* Common symbol.  */
+  COFF_SYMBOL_COMMON,
+   /* Undefined symbol.  */
+  COFF_SYMBOL_UNDEFINED,
+   /* Local symbol.  */
+  COFF_SYMBOL_LOCAL,
+   /* PE section symbol.  */
+  COFF_SYMBOL_PE_SECTION
+};
+
 typedef struct
 {
   void (*_bfd_coff_swap_aux_in) PARAMS ((
@@ -583,43 +664,43 @@ typedef struct
        PTR             in));
 
  unsigned int (*_bfd_coff_swap_aux_out) PARAMS ((
-       bfd   	*abfd,
-       PTR	in,
-       int    	type,
-       int    	class,
+       bfd     *abfd,
+       PTR     in,
+       int     type,
+       int     class,
        int     indaux,
        int     numaux,
-       PTR    	ext));
+       PTR     ext));
 
  unsigned int (*_bfd_coff_swap_sym_out) PARAMS ((
       bfd      *abfd,
-      PTR	in,
-      PTR	ext));
+      PTR      in,
+      PTR      ext));
 
  unsigned int (*_bfd_coff_swap_lineno_out) PARAMS ((
-      	bfd   	*abfd,
-      	PTR	in,
-	PTR	ext));
+       bfd     *abfd,
+       PTR     in,
+       PTR     ext));
 
  unsigned int (*_bfd_coff_swap_reloc_out) PARAMS ((
-      	bfd     *abfd,
-     	PTR	src,
-	PTR	dst));
+       bfd     *abfd,
+       PTR     src,
+       PTR     dst));
 
  unsigned int (*_bfd_coff_swap_filehdr_out) PARAMS ((
-      	bfd  	*abfd,
-	PTR 	in,
-	PTR 	out));
+       bfd     *abfd,
+       PTR     in,
+       PTR     out));
 
  unsigned int (*_bfd_coff_swap_aouthdr_out) PARAMS ((
-      	bfd 	*abfd,
-	PTR 	in,
-	PTR	out));
+       bfd     *abfd,
+       PTR     in,
+       PTR     out));
 
  unsigned int (*_bfd_coff_swap_scnhdr_out) PARAMS ((
-      	bfd  	*abfd,
-      	PTR	in,
-	PTR	out));
+       bfd     *abfd,
+       PTR     in,
+       PTR     out));
 
  unsigned int _bfd_filhsz;
  unsigned int _bfd_aoutsz;
@@ -628,8 +709,12 @@ typedef struct
  unsigned int _bfd_auxesz;
  unsigned int _bfd_relsz;
  unsigned int _bfd_linesz;
+ unsigned int _bfd_filnmlen;
  boolean _bfd_coff_long_filenames;
  boolean _bfd_coff_long_section_names;
+ unsigned int _bfd_coff_default_section_alignment_power;
+ boolean _bfd_coff_force_symnames_in_strings;
+ unsigned int _bfd_coff_debug_string_prefix_length;
  void (*_bfd_coff_swap_filehdr_in) PARAMS ((
        bfd     *abfd,
        PTR     ext,
@@ -659,7 +744,8 @@ typedef struct
  flagword (*_bfd_styp_to_sec_flags_hook) PARAMS ((
        bfd     *abfd,
        PTR     internal_scnhdr,
-       const char *name));
+       const char *name,
+       asection *section));
  void (*_bfd_set_alignment_hook) PARAMS ((
        bfd     *abfd,
        asection *sec,
@@ -696,10 +782,10 @@ typedef struct
        arelent *r,
        unsigned int shrink,
        struct bfd_link_info *link_info));
- boolean (*_bfd_coff_sym_is_global) PARAMS ((
+ enum coff_symbol_classification (*_bfd_coff_classify_symbol) PARAMS ((
        bfd *abfd,
        struct internal_syment *));
- void (*_bfd_coff_compute_section_file_positions) PARAMS ((
+ boolean (*_bfd_coff_compute_section_file_positions) PARAMS ((
        bfd *abfd));
  boolean (*_bfd_coff_start_final_link) PARAMS ((
        bfd *output_bfd,
@@ -739,6 +825,13 @@ typedef struct
        boolean collect, 
        struct bfd_link_hash_entry **hashp));
 
+ boolean (*_bfd_coff_link_output_has_begun) PARAMS ((
+       bfd * abfd,
+       struct coff_final_link_info * pfinfo));
+ boolean (*_bfd_coff_final_link_postscript) PARAMS ((
+       bfd * abfd,
+       struct coff_final_link_info * pfinfo));
+
 } bfd_coff_backend_data;
 
 #define coff_backend_info(abfd) ((bfd_coff_backend_data *) (abfd)->xvec->backend_data)
@@ -780,9 +873,12 @@ typedef struct
 #define bfd_coff_auxesz(abfd) (coff_backend_info (abfd)->_bfd_auxesz)
 #define bfd_coff_relsz(abfd)  (coff_backend_info (abfd)->_bfd_relsz)
 #define bfd_coff_linesz(abfd) (coff_backend_info (abfd)->_bfd_linesz)
+#define bfd_coff_filnmlen(abfd) (coff_backend_info (abfd)->_bfd_filnmlen)
 #define bfd_coff_long_filenames(abfd) (coff_backend_info (abfd)->_bfd_coff_long_filenames)
 #define bfd_coff_long_section_names(abfd) \
         (coff_backend_info (abfd)->_bfd_coff_long_section_names)
+#define bfd_coff_default_section_alignment_power(abfd) \
+        (coff_backend_info (abfd)->_bfd_coff_default_section_alignment_power)
 #define bfd_coff_swap_filehdr_in(abfd, i,o) \
         ((coff_backend_info (abfd)->_bfd_coff_swap_filehdr_in) (abfd, i, o))
 
@@ -803,8 +899,9 @@ typedef struct
 #define bfd_coff_mkobject_hook(abfd, filehdr, aouthdr)\
         ((coff_backend_info (abfd)->_bfd_coff_mkobject_hook) (abfd, filehdr, aouthdr))
 
-#define bfd_coff_styp_to_sec_flags_hook(abfd, scnhdr, name)\
-        ((coff_backend_info (abfd)->_bfd_styp_to_sec_flags_hook) (abfd, scnhdr, name))
+#define bfd_coff_styp_to_sec_flags_hook(abfd, scnhdr, name, section)\
+        ((coff_backend_info (abfd)->_bfd_styp_to_sec_flags_hook)\
+         (abfd, scnhdr, name, section))
 
 #define bfd_coff_set_alignment_hook(abfd, sec, scnhdr)\
         ((coff_backend_info (abfd)->_bfd_set_alignment_hook) (abfd, sec, scnhdr))
@@ -815,6 +912,12 @@ typedef struct
 #define bfd_coff_symname_in_debug(abfd, sym)\
         ((coff_backend_info (abfd)->_bfd_coff_symname_in_debug) (abfd, sym))
 
+#define bfd_coff_force_symnames_in_strings(abfd)\
+       (coff_backend_info (abfd)->_bfd_coff_force_symnames_in_strings)
+
+#define bfd_coff_debug_string_prefix_length(abfd)\
+       (coff_backend_info (abfd)->_bfd_coff_debug_string_prefix_length)
+
 #define bfd_coff_print_aux(abfd, file, base, symbol, aux, indaux)\
         ((coff_backend_info (abfd)->_bfd_coff_print_aux)\
          (abfd, file, base, symbol, aux, indaux))
@@ -827,8 +930,8 @@ typedef struct
         ((coff_backend_info (abfd)->_bfd_coff_reloc16_estimate)\
          (abfd, section, reloc, shrink, link_info))
 
-#define bfd_coff_sym_is_global(abfd, sym)\
-        ((coff_backend_info (abfd)->_bfd_coff_sym_is_global)\
+#define bfd_coff_classify_symbol(abfd, sym)\
+        ((coff_backend_info (abfd)->_bfd_coff_classify_symbol)\
          (abfd, sym))
 
 #define bfd_coff_compute_section_file_positions(abfd)\
@@ -851,3 +954,8 @@ typedef struct
         ((coff_backend_info (abfd)->_bfd_coff_link_add_one_symbol)\
          (info, abfd, name, flags, section, value, string, cp, coll, hashp))
 
+#define bfd_coff_link_output_has_begun(a,p) \
+        ((coff_backend_info (a)->_bfd_coff_link_output_has_begun) (a,p))
+#define bfd_coff_final_link_postscript(a,p) \
+        ((coff_backend_info (a)->_bfd_coff_final_link_postscript) (a,p))
+
