@@ -1,5 +1,5 @@
 /* Support for the generic parts of PE/PEI; the common executable parts.
-   Copyright 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
+   Copyright 1995, 96, 97, 98, 99, 2000 Free Software Foundation, Inc.
    Written by Cygnus Solutions.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -434,11 +434,14 @@ _bfd_pei_swap_aouthdr_in (abfd, aouthdr_ext1, aouthdr_int1)
     GET_AOUTHDR_ENTRY (abfd, (bfd_byte *) aouthdr_ext->entry);
   aouthdr_int->text_start =
     GET_AOUTHDR_TEXT_START (abfd, (bfd_byte *) aouthdr_ext->text_start);
+#ifndef COFF_WITH_PEP64
+  /* PE32+ does not have data_start member! */
   aouthdr_int->data_start =
     GET_AOUTHDR_DATA_START (abfd, (bfd_byte *) aouthdr_ext->data_start);
+#endif
 
   a = &aouthdr_int->pe;
-  a->ImageBase = bfd_h_get_32 (abfd, (bfd_byte *)src->ImageBase);
+  a->ImageBase = GET_OPTHDR_IMAGE_BASE (abfd, (bfd_byte *)src->ImageBase);
   a->SectionAlignment = bfd_h_get_32 (abfd, (bfd_byte *)src->SectionAlignment);
   a->FileAlignment = bfd_h_get_32 (abfd, (bfd_byte *)src->FileAlignment);
   a->MajorOperatingSystemVersion =
@@ -455,10 +458,10 @@ _bfd_pei_swap_aouthdr_in (abfd, aouthdr_ext1, aouthdr_int1)
   a->CheckSum = bfd_h_get_32 (abfd, (bfd_byte *)src->CheckSum);
   a->Subsystem = bfd_h_get_16 (abfd, (bfd_byte *)src->Subsystem);
   a->DllCharacteristics = bfd_h_get_16 (abfd, (bfd_byte *)src->DllCharacteristics);
-  a->SizeOfStackReserve = bfd_h_get_32 (abfd, (bfd_byte *)src->SizeOfStackReserve);
-  a->SizeOfStackCommit = bfd_h_get_32 (abfd, (bfd_byte *)src->SizeOfStackCommit);
-  a->SizeOfHeapReserve = bfd_h_get_32 (abfd, (bfd_byte *)src->SizeOfHeapReserve);
-  a->SizeOfHeapCommit = bfd_h_get_32 (abfd, (bfd_byte *)src->SizeOfHeapCommit);
+  a->SizeOfStackReserve = GET_OPTHDR_SIZE_OF_STACK_RESERVE (abfd, (bfd_byte *)src->SizeOfStackReserve);
+  a->SizeOfStackCommit = GET_OPTHDR_SIZE_OF_STACK_COMMIT (abfd, (bfd_byte *)src->SizeOfStackCommit);
+  a->SizeOfHeapReserve = GET_OPTHDR_SIZE_OF_HEAP_RESERVE (abfd, (bfd_byte *)src->SizeOfHeapReserve);
+  a->SizeOfHeapCommit = GET_OPTHDR_SIZE_OF_HEAP_COMMIT (abfd, (bfd_byte *)src->SizeOfHeapCommit);
   a->LoaderFlags = bfd_h_get_32 (abfd, (bfd_byte *)src->LoaderFlags);
   a->NumberOfRvaAndSizes = bfd_h_get_32 (abfd, (bfd_byte *)src->NumberOfRvaAndSizes);
 
@@ -476,18 +479,25 @@ _bfd_pei_swap_aouthdr_in (abfd, aouthdr_ext1, aouthdr_int1)
   if (aouthdr_int->entry)
     {
       aouthdr_int->entry += a->ImageBase;
+#ifndef COFF_WITH_PEP64
       aouthdr_int->entry &= 0xffffffff;
+#endif
     }
   if (aouthdr_int->tsize) 
     {
       aouthdr_int->text_start += a->ImageBase;
+#ifndef COFF_WITH_PEP64
       aouthdr_int->text_start &= 0xffffffff;
+#endif
     }
+#ifndef COFF_WITH_PEP64
+  /* PE32+ does not have data_start member! */
   if (aouthdr_int->dsize) 
     {
       aouthdr_int->data_start += a->ImageBase;
       aouthdr_int->data_start &= 0xffffffff;
     }
+#endif
 
 #ifdef POWERPC_LE_PE
   /* These three fields are normally set up by ppc_relocate_section.
@@ -532,10 +542,28 @@ _bfd_pei_swap_aouthdr_out (abfd, in, out)
   struct internal_aouthdr *aouthdr_in = (struct internal_aouthdr *)in;
   struct internal_extra_pe_aouthdr *extra = &pe_data (abfd)->pe_opthdr;
   PEAOUTHDR *aouthdr_out = (PEAOUTHDR *)out;
+  bfd_vma sa, fa, ib;
 
-  bfd_vma sa = extra->SectionAlignment;
-  bfd_vma fa = extra->FileAlignment;
-  bfd_vma ib = extra->ImageBase ;
+  /* The following definitely is required for EFI applications.
+     Perhaps it's needed for other PEI targets as well, but I don't
+     know that for a fact, so we play it safe here and tweak the
+     alignments only if PEI_FORCE_MINIMUM_ALIGNMENT is
+     defined. --davidm */
+#ifdef PEI_FORCE_MINIMUM_ALIGNMENT
+  if (!extra->FileAlignment)
+    extra->FileAlignment = PE_DEF_FILE_ALIGNMENT;
+  if (!extra->SectionAlignment)
+    extra->SectionAlignment = PE_DEF_SECTION_ALIGNMENT;
+#endif
+
+#ifdef PEI_DEFAULT_TARGET_SUBSYSTEM
+  if (extra->Subsystem == IMAGE_SUBSYSTEM_UNKNOWN)
+    extra->Subsystem = PEI_DEFAULT_TARGET_SUBSYSTEM;
+#endif
+
+  sa = extra->SectionAlignment;
+  fa = extra->FileAlignment;
+  ib = extra->ImageBase;
 
   if (aouthdr_in->tsize) 
     {
@@ -566,7 +594,7 @@ _bfd_pei_swap_aouthdr_out (abfd, in, out)
   /* first null out all data directory entries .. */
   memset (extra->DataDirectory, sizeof (extra->DataDirectory), 0);
 
-  add_data_entry (abfd, extra, 0, ".edata", 0);
+  add_data_entry (abfd, extra, 0, ".edata", ib);
 
   /* Don't call add_data_entry for .idata$2 or .idata$5.  It's done in
      bfd_coff_final_link where all the required information is
@@ -574,11 +602,11 @@ _bfd_pei_swap_aouthdr_out (abfd, in, out)
 
   /* However, until other .idata fixes are made (pending patch), the
      entry for .idata is needed for backwards compatability.  FIXME.  */
-  add_data_entry (abfd, extra, 1, ".idata" ,0);
+  add_data_entry (abfd, extra, 1, ".idata" , ib);
 
-  add_data_entry (abfd, extra, 2, ".rsrc" ,0);
+  add_data_entry (abfd, extra, 2, ".rsrc" , ib);
 
-  add_data_entry (abfd, extra, 3, ".pdata", 0);
+  add_data_entry (abfd, extra, 3, ".pdata", ib);
 
   /* For some reason, the virtual size (which is what's set by
      add_data_entry) for .reloc is not the same as the size recorded
@@ -586,7 +614,7 @@ _bfd_pei_swap_aouthdr_out (abfd, in, out)
      but since it's the best we've got, use it.  It does do the right
      thing for .pdata.  */
   if (pe_data (abfd)->has_reloc_section)
-    add_data_entry (abfd, extra, 5, ".reloc", 0);
+    add_data_entry (abfd, extra, 5, ".reloc", ib);
 
   {
     asection *sec;
@@ -633,12 +661,14 @@ _bfd_pei_swap_aouthdr_out (abfd, in, out)
   PUT_AOUTHDR_TEXT_START (abfd, aouthdr_in->text_start,
 			  (bfd_byte *) aouthdr_out->standard.text_start);
 
+#ifndef COFF_WITH_PEP64
+  /* PE32+ does not have data_start member! */
   PUT_AOUTHDR_DATA_START (abfd, aouthdr_in->data_start,
 			  (bfd_byte *) aouthdr_out->standard.data_start);
+#endif
 
-
-  bfd_h_put_32 (abfd, extra->ImageBase,
-		(bfd_byte *) aouthdr_out->ImageBase);
+  PUT_OPTHDR_IMAGE_BASE (abfd, extra->ImageBase,
+			 (bfd_byte *) aouthdr_out->ImageBase);
   bfd_h_put_32 (abfd, extra->SectionAlignment,
 		(bfd_byte *) aouthdr_out->SectionAlignment);
   bfd_h_put_32 (abfd, extra->FileAlignment,
@@ -667,14 +697,14 @@ _bfd_pei_swap_aouthdr_out (abfd, in, out)
 		(bfd_byte *) aouthdr_out->Subsystem);
   bfd_h_put_16 (abfd, extra->DllCharacteristics,
 		(bfd_byte *) aouthdr_out->DllCharacteristics);
-  bfd_h_put_32 (abfd, extra->SizeOfStackReserve,
-		(bfd_byte *) aouthdr_out->SizeOfStackReserve);
-  bfd_h_put_32 (abfd, extra->SizeOfStackCommit,
-		(bfd_byte *) aouthdr_out->SizeOfStackCommit);
-  bfd_h_put_32 (abfd, extra->SizeOfHeapReserve,
-		(bfd_byte *) aouthdr_out->SizeOfHeapReserve);
-  bfd_h_put_32 (abfd, extra->SizeOfHeapCommit,
-		(bfd_byte *) aouthdr_out->SizeOfHeapCommit);
+  PUT_OPTHDR_SIZE_OF_STACK_RESERVE (abfd, extra->SizeOfStackReserve,
+				    (bfd_byte *) aouthdr_out->SizeOfStackReserve);
+  PUT_OPTHDR_SIZE_OF_STACK_COMMIT (abfd, extra->SizeOfStackCommit,
+				   (bfd_byte *) aouthdr_out->SizeOfStackCommit);
+  PUT_OPTHDR_SIZE_OF_HEAP_RESERVE (abfd, extra->SizeOfHeapReserve,
+				   (bfd_byte *) aouthdr_out->SizeOfHeapReserve);
+  PUT_OPTHDR_SIZE_OF_HEAP_COMMIT (abfd, extra->SizeOfHeapCommit,
+				  (bfd_byte *) aouthdr_out->SizeOfHeapCommit);
   bfd_h_put_32 (abfd, extra->LoaderFlags,
 		(bfd_byte *) aouthdr_out->LoaderFlags);
   bfd_h_put_32 (abfd, extra->NumberOfRvaAndSizes,
@@ -973,7 +1003,7 @@ static char * dir_names[IMAGE_NUMBEROF_DIRECTORY_ENTRIES] =
   N_ ("Load Configuration Directory"),
   N_ ("Bound Import Directory"),
   N_ ("Import Address Table Directory"),
-  N_ ("Reserved"),
+  N_ ("Delay Import Directory"),
   N_ ("Reserved"),
   N_ ("Reserved")
 };
@@ -994,64 +1024,62 @@ pe_print_idata (abfd, vfile)
      PTR vfile;
 {
   FILE *file = (FILE *) vfile;
-  bfd_byte *data = 0;
-  asection *section = bfd_get_section_by_name (abfd, ".idata");
-  unsigned long adj;
+  bfd_byte *data;
+  asection *section;
+  bfd_signed_vma adj;
 
 #ifdef POWERPC_LE_PE
   asection *rel_section = bfd_get_section_by_name (abfd, ".reldata");
 #endif
 
-  bfd_size_type datasize;
+  bfd_size_type datasize = 0;
   bfd_size_type dataoff;
-  bfd_size_type secsize;
   bfd_size_type i;
   int onaline = 20;
 
   pe_data_type *pe = pe_data (abfd);
   struct internal_extra_pe_aouthdr *extra = &pe->pe_opthdr;
 
-  if (section != NULL)
-    {
-      datasize = bfd_section_size (abfd, section);
-      dataoff = 0;
+  bfd_vma addr;
 
-      if (datasize == 0)
+  addr = extra->DataDirectory[1].VirtualAddress;
+
+  if (addr == 0 && extra->DataDirectory[1].Size == 0)
+    {
+      /* Maybe the extra header isn't there.  Look for the section.  */
+      section = bfd_get_section_by_name (abfd, ".idata");
+      if (section == NULL)
 	return true;
 
-      fprintf (file, _("\nThe import table is the .idata section\n"));
+      addr = section->vma;
+      datasize = bfd_section_size (abfd, section);
+      if (datasize == 0)
+	return true;
     }
   else
     {
-      /* idata buried in some other section: e.g. KERNEL32.DLL.  */
-      bfd_vma addr, size;
-
-      addr = extra->DataDirectory[1].VirtualAddress;
-      size = extra->DataDirectory[1].Size;
-
-      if (addr == 0 || size == 0)
-	return true;
-
+      addr += extra->ImageBase;
       for (section = abfd->sections; section != NULL; section = section->next)
 	{
-	   if (addr >= section->vma
-	       && addr < section->vma + bfd_section_size(abfd,section))
-	         break;
+	  datasize = bfd_section_size (abfd, section);
+	  if (addr >= section->vma && addr < section->vma + datasize)
+	    break;
 	}
+
       if (section == NULL)
 	{
-	   fprintf (file,
-		    _("\nThere is an import table, but the section containing it could not be found\n"));
-	   return true;
+	  fprintf (file,
+		   _("\nThere is an import table, but the section containing it could not be found\n"));
+	  return true;
 	}
-
-      fprintf (file, _("\nThere is an import table in %s at 0x%lx\n"),
-	       section->name, (unsigned long)addr);
-
-      dataoff = addr - section->vma;
-      datasize = size;
     }
 
+  fprintf (file, _("\nThere is an import table in %s at 0x%lx\n"),
+	   section->name, (unsigned long) addr);
+
+  dataoff = addr - section->vma;
+  datasize -= dataoff;
+
 #ifdef POWERPC_LE_PE
   if (rel_section != 0 && bfd_section_size (abfd, rel_section) != 0)
     {
@@ -1067,13 +1095,12 @@ pe_print_idata (abfd, vfile)
       bfd_vma start_address;
       bfd_byte *data = 0;
       int offset;
+
       data = (bfd_byte *) bfd_malloc ((size_t) bfd_section_size (abfd,
 								 rel_section));
       if (data == NULL && bfd_section_size (abfd, rel_section) != 0)
 	return false;
 
-      datasize = bfd_section_size (abfd, rel_section);
-  
       bfd_get_section_contents (abfd,
 				rel_section,
 				(PTR) data, 0,
@@ -1100,21 +1127,21 @@ pe_print_idata (abfd, vfile)
 #endif
 
   fprintf(file,
-	  _("\nThe Import Tables (interpreted .idata section contents)\n"));
+	  _("\nThe Import Tables (interpreted %s section contents)\n"),
+	  section->name);
   fprintf(file,
 	  _(" vma:            Hint    Time      Forward  DLL       First\n"));
   fprintf(file,
 	  _("                 Table   Stamp     Chain    Name      Thunk\n"));
 
-  secsize = bfd_section_size (abfd, section);
-  data = (bfd_byte *) bfd_malloc (secsize);
-  if (data == NULL && secsize != 0)
+  data = (bfd_byte *) bfd_malloc (datasize);
+  if (data == NULL)
     return false;
 
-  if (! bfd_get_section_contents (abfd, section, (PTR) data, 0, secsize))
+  if (! bfd_get_section_contents (abfd, section, (PTR) data, dataoff, datasize))
     return false;
 
-  adj = - section->vma;
+  adj = section->vma - extra->ImageBase + dataoff;
 
   for (i = 0; i < datasize; i += onaline)
     {
@@ -1127,40 +1154,39 @@ pe_print_idata (abfd, vfile)
       bfd_size_type j;
       char *dll;
 
-      fprintf (file,
-	       " %08lx\t",
-	       (unsigned long int) (i + section->vma + dataoff));
-      
+      /* print (i + extra->DataDirectory[1].VirtualAddress)  */
+      fprintf (file, " %08lx\t", (unsigned long) (i + adj));
+
       if (i + 20 > datasize)
 	{
 	  /* check stuff */
 	  ;
 	}
 
-      hint_addr = bfd_get_32 (abfd, data + i + dataoff);
-      time_stamp = bfd_get_32 (abfd, data + i + 4 + dataoff);
-      forward_chain = bfd_get_32 (abfd, data + i + 8 + dataoff);
-      dll_name = bfd_get_32 (abfd, data + i + 12 + dataoff);
-      first_thunk = bfd_get_32 (abfd, data + i + 16 + dataoff);
+      hint_addr = bfd_get_32 (abfd, data + i);
+      time_stamp = bfd_get_32 (abfd, data + i + 4);
+      forward_chain = bfd_get_32 (abfd, data + i + 8);
+      dll_name = bfd_get_32 (abfd, data + i + 12);
+      first_thunk = bfd_get_32 (abfd, data + i + 16);
 
       fprintf (file, "%08lx %08lx %08lx %08lx %08lx\n",
-	       hint_addr,
-	       time_stamp,
-	       forward_chain,
-	       dll_name,
-	       first_thunk);
+	       (unsigned long) hint_addr,
+	       (unsigned long) time_stamp,
+	       (unsigned long) forward_chain,
+	       (unsigned long) dll_name,
+	       (unsigned long) first_thunk);
 
       if (hint_addr == 0 && first_thunk == 0)
 	break;
 
-      dll = (char *) data + dll_name - section->vma + dataoff;
+      dll = (char *) data + dll_name - adj;
       fprintf(file, _("\n\tDLL Name: %s\n"), dll);
 
       if (hint_addr != 0)
 	{
 	  fprintf (file, _("\tvma:  Hint/Ord Member-Name\n"));
 
-	  idx = hint_addr + adj;
+	  idx = hint_addr - adj;
 
 	  for (j = 0; j < datasize; j += 4)
 	    {
@@ -1176,8 +1202,8 @@ pe_print_idata (abfd, vfile)
 		  int ordinal;
 		  char *member_name;
 
-		  ordinal = bfd_get_16 (abfd, data + member + adj);
-		  member_name = (char *) data + member + adj + 2;
+		  ordinal = bfd_get_16 (abfd, data + member - adj);
+		  member_name = (char *) data + member - adj + 2;
 		  fprintf (file, "\t%04lx\t %4d  %s",
 			   member, ordinal, member_name);
 		}
@@ -1188,7 +1214,7 @@ pe_print_idata (abfd, vfile)
 		  && first_thunk != 0
 		  && first_thunk != hint_addr)
 		fprintf (file, "\t%04lx",
-			 bfd_get_32 (abfd, data + first_thunk + adj + j));
+			 (long) bfd_get_32 (abfd, data + first_thunk - adj + j));
 
 	      fprintf (file, "\n");
 	    }
@@ -1199,7 +1225,7 @@ pe_print_idata (abfd, vfile)
 	  int differ = 0;
 	  int idx2;
 
-	  idx2 = first_thunk + adj;
+	  idx2 = first_thunk - adj;
 
 	  for (j = 0; j < datasize; j += 4)
 	    {
@@ -1232,10 +1258,12 @@ pe_print_idata (abfd, vfile)
 		  else
 		    {
 		      ordinal = bfd_get_16(abfd,
-					   data + iat_member + adj);
-		      member_name = (char *) data + iat_member + adj + 2;
+					   data + iat_member - adj);
+		      member_name = (char *) data + iat_member - adj + 2;
 		      fprintf(file, "\t%04lx\t %4d  %s\n",
-			      iat_member, ordinal, member_name);
+			      (unsigned long) iat_member,
+			      ordinal,
+			      member_name);
 		    }
 		}
 
@@ -1264,14 +1292,14 @@ pe_print_edata (abfd, vfile)
      PTR vfile;
 {
   FILE *file = (FILE *) vfile;
-  bfd_byte *data = 0;
-  asection *section = bfd_get_section_by_name (abfd, ".edata");
+  bfd_byte *data;
+  asection *section;
 
-  bfd_size_type datasize;
+  bfd_size_type datasize = 0;
   bfd_size_type dataoff;
   bfd_size_type i;
 
-  int adj;
+  bfd_signed_vma adj;
   struct EDT_type
     {
       long export_flags;             /* reserved - should be zero */
@@ -1290,45 +1318,48 @@ pe_print_edata (abfd, vfile)
   pe_data_type *pe = pe_data (abfd);
   struct internal_extra_pe_aouthdr *extra = &pe->pe_opthdr;
 
-  if (section != NULL)
+  bfd_vma addr;
+
+  addr = extra->DataDirectory[0].VirtualAddress;
+
+  if (addr == 0 && extra->DataDirectory[0].Size == 0)
     {
+      /* Maybe the extra header isn't there.  Look for the section.  */
+      section = bfd_get_section_by_name (abfd, ".edata");
+      if (section == NULL)
+	return true;
+
+      addr = section->vma;
       datasize = bfd_section_size (abfd, section);
-      dataoff = 0;
-      fprintf (file, _("\nThe export table is the .edata section\n"));
+      if (datasize == 0)
+	return true;
     }
   else
     {
-      /* edata is buried in some other section: e.g. NTDLL.DLL.  */
-      bfd_vma addr, size;
-
-      addr = extra->DataDirectory[0].VirtualAddress;
-      size = extra->DataDirectory[0].Size;
-
-      if (addr == 0 || size == 0)
-	return true;
-
+      addr += extra->ImageBase;
       for (section = abfd->sections; section != NULL; section = section->next)
 	{
-	   if (addr >= section->vma
-	       && addr < section->vma + bfd_section_size (abfd, section))
-	     break;
+	  datasize = bfd_section_size (abfd, section);
+	  if (addr >= section->vma && addr < section->vma + datasize)
+	    break;
 	}
+
       if (section == NULL)
 	{
-	   fprintf (file,
-		    _("\nThere is an export table, but the section containing it could not be found\n"));
-	   return true;
+	  fprintf (file,
+		   _("\nThere is an export table, but the section containing it could not be found\n"));
+	  return true;
 	}
-
-      fprintf (file, _("\nThere is an export table in %s at 0x%lx\n"),
-	       section->name, (unsigned long) addr);
-
-      datasize = size;
-      dataoff = addr - section->vma;
     }
 
+  fprintf (file, _("\nThere is an export table in %s at 0x%lx\n"),
+	   section->name, (unsigned long) addr);
+
+  dataoff = addr - section->vma;
+  datasize -= dataoff;
+
   data = (bfd_byte *) bfd_malloc (datasize);
-  if (data == NULL && datasize != 0)
+  if (data == NULL)
     return false;
 
   if (! bfd_get_section_contents (abfd, section, (PTR) data, dataoff,
@@ -1348,11 +1379,12 @@ pe_print_edata (abfd, vfile)
   edt.npt_addr       = bfd_get_32(abfd, data+32);
   edt.ot_addr        = bfd_get_32(abfd, data+36);
 
-  adj = - (section->vma + dataoff);
+  adj = section->vma - extra->ImageBase + dataoff;
 
   /* Dump the EDT first first */
   fprintf(file,
-	  _("\nThe Export Tables (interpreted .edata section contents)\n\n"));
+	  _("\nThe Export Tables (interpreted %s section contents)\n\n"),
+	  section->name);
 
   fprintf(file,
 	  _("Export Flags \t\t\t%lx\n"), (unsigned long) edt.export_flags);
@@ -1367,7 +1399,7 @@ pe_print_edata (abfd, vfile)
 	   _("Name \t\t\t\t"));
   fprintf_vma (file, edt.name);
   fprintf (file,
-	   " %s\n", data + edt.name + adj);
+	   " %s\n", data + edt.name - adj);
 
   fprintf(file,
 	  _("Ordinal Base \t\t\t%ld\n"), edt.base);
@@ -1376,11 +1408,11 @@ pe_print_edata (abfd, vfile)
 	  _("Number in:\n"));
 
   fprintf(file,
-	  _("\tExport Address Table \t\t%lx\n"),
+	  _("\tExport Address Table \t\t%08lx\n"),
 	  edt.num_functions);
 
   fprintf(file,
-	  _("\t[Name Pointer/Ordinal] Table\t%lu\n"), edt.num_names);
+	  _("\t[Name Pointer/Ordinal] Table\t%08lx\n"), edt.num_names);
 
   fprintf(file,
 	  _("Table Addresses\n"));
@@ -1418,7 +1450,7 @@ pe_print_edata (abfd, vfile)
   for (i = 0; i < edt.num_functions; ++i)
     {
       bfd_vma eat_member = bfd_get_32 (abfd,
-				       data + edt.eat_addr + (i * 4) + adj);
+				       data + edt.eat_addr + (i * 4) - adj);
       bfd_vma eat_actual = eat_member;
       bfd_vma edata_start = bfd_get_section_vma (abfd, section);
       bfd_vma edata_end = edata_start + datasize;
@@ -1432,15 +1464,20 @@ pe_print_edata (abfd, vfile)
 	  /* Should locate a function descriptor */
 	  fprintf (file,
 		   "\t[%4ld] +base[%4ld] %04lx %s -- %s\n",
-		   (long) i, (long) (i + edt.base), eat_member,
-		   _("Forwarder RVA"), data + eat_member + adj);
+		   (long) i,
+		   (long) (i + edt.base),
+		   (unsigned long) eat_member,
+		   _("Forwarder RVA"),
+		   data + eat_member - adj);
 	}
       else
 	{
 	  /* Should locate a function descriptor in the reldata section */
 	  fprintf (file,
 		   "\t[%4ld] +base[%4ld] %04lx %s\n",
-		   (long) i, (long) (i + edt.base), eat_member,
+		   (long) i,
+		   (long) (i + edt.base),
+		   (unsigned long) eat_member,
 		   _("Export RVA"));
 	}
     }
@@ -1455,14 +1492,14 @@ pe_print_edata (abfd, vfile)
       bfd_vma name_ptr = bfd_get_32(abfd,
 				    data +
 				    edt.npt_addr
-				    + (i*4) + adj);
+				    + (i*4) - adj);
       
-      char *name = (char *) data + name_ptr + adj;
+      char *name = (char *) data + name_ptr - adj;
 
       bfd_vma ord = bfd_get_16(abfd,
 				    data +
 				    edt.ot_addr
-				    + (i*2) + adj);
+				    + (i*2) - adj);
       fprintf(file,
 	      "\t[%4ld] %s\n", (long) ord, name);
 
@@ -1473,18 +1510,27 @@ pe_print_edata (abfd, vfile)
   return true;
 }
 
+/* This really is architecture dependent.  On IA-64, a .pdata entry
+   consists of three dwords containing relative virtual addresses that
+   specify the start and end address of the code range the entry
+   covers and the address of the corresponding unwind info data.  */
 static boolean
 pe_print_pdata (abfd, vfile)
      bfd  *abfd;
      PTR vfile;
 {
+#ifdef COFF_WITH_PEP64
+# define PDATA_ROW_SIZE	(3*8)
+#else
+# define PDATA_ROW_SIZE	(5*4)
+#endif
   FILE *file = (FILE *) vfile;
   bfd_byte *data = 0;
   asection *section = bfd_get_section_by_name (abfd, ".pdata");
   bfd_size_type datasize = 0;
   bfd_size_type i;
   bfd_size_type start, stop;
-  int onaline = 20;
+  int onaline = PDATA_ROW_SIZE;
 
   if (section == NULL
       || coff_section_data (abfd, section) == NULL
@@ -1498,10 +1544,15 @@ pe_print_pdata (abfd, vfile)
 
   fprintf (file,
 	   _("\nThe Function Table (interpreted .pdata section contents)\n"));
+#ifdef COFF_WITH_PEP64
+  fprintf(file,
+	  _(" vma:\t\t\tBegin Address    End Address      Unwind Info\n"));
+#else
   fprintf (file,
 	   _(" vma:\t\tBegin    End      EH       EH       PrologEnd  Exception\n"));
   fprintf (file,
 	   _("     \t\tAddress  Address  Handler  Data     Address    Mask\n"));
+#endif
 
   if (bfd_section_size (abfd, section) == 0)
     return true;
@@ -1527,14 +1578,14 @@ pe_print_pdata (abfd, vfile)
       bfd_vma prolog_end_addr;
       int em_data;
 
-      if (i + 20 > stop)
+      if (i + PDATA_ROW_SIZE > stop)
 	break;
 
-      begin_addr = bfd_get_32(abfd, data+i);
-      end_addr = bfd_get_32(abfd, data+i+4);
-      eh_handler = bfd_get_32(abfd, data+i+8);
-      eh_data = bfd_get_32(abfd, data+i+12);
-      prolog_end_addr = bfd_get_32(abfd, data+i+16);
+      begin_addr = GET_PDATA_ENTRY(abfd, data+i);
+      end_addr = GET_PDATA_ENTRY(abfd, data+i+4);
+      eh_handler = GET_PDATA_ENTRY(abfd, data+i+8);
+      eh_data = GET_PDATA_ENTRY(abfd, data+i+12);
+      prolog_end_addr = GET_PDATA_ENTRY(abfd, data+i+16);
       
       if (begin_addr == 0 && end_addr == 0 && eh_handler == 0
 	  && eh_data == 0 && prolog_end_addr == 0)
@@ -1543,21 +1594,21 @@ pe_print_pdata (abfd, vfile)
 	  break;
 	}
 
-      fprintf (file,
-	       " %08lx\t",
-	       (unsigned long int) (i + section->vma));
-
       em_data = ((eh_handler & 0x1) << 2) | (prolog_end_addr & 0x3);
-      eh_handler &= 0xfffffffc;
-      prolog_end_addr &= 0xfffffffc;
+      eh_handler &= ~ (bfd_vma) 0x3;
+      prolog_end_addr &= ~ (bfd_vma) 0x3;
 
-      fprintf (file, "%08lx %08lx %08lx %08lx %08lx   %x",
-	       begin_addr,
-	       end_addr,
-	       eh_handler,
-	       eh_data,
-	       prolog_end_addr,
-	       em_data);
+      fputc (' ', file);
+      fprintf_vma (file, i + section->vma); fputc ('\t', file);
+      fprintf_vma (file, begin_addr); fputc (' ', file);
+      fprintf_vma (file, end_addr); fputc (' ', file);
+      fprintf_vma (file, eh_handler);
+#ifndef COFF_WITH_PEP64
+      fputc (' ', file);
+      fprintf_vma (file, eh_data); fputc (' ', file);
+      fprintf_vma (file, prolog_end_addr);
+      fprintf (file, "   %x", em_data);
+#endif
 
 #ifdef POWERPC_LE_PE
       if (eh_handler == 0 && eh_data != 0)
@@ -1603,6 +1654,12 @@ static const char * const tbl[] =
 "HIGHLOW",
 "HIGHADJ",
 "MIPS_JMPADDR",
+"SECTION",
+"REL32",
+"RESERVED1",
+"MIPS_JMPADDR16",
+"DIR64",
+"HIGH3ADJ"
 "UNKNOWN",   /* MUST be last */
 };
 
@@ -1661,7 +1718,7 @@ pe_print_reloc (abfd, vfile)
 
       fprintf (file,
 	       _("\nVirtual Address: %08lx Chunk size %ld (0x%lx) Number of fixups %ld\n"),
-	       virtual_address, size, size, number);
+	       (unsigned long) virtual_address, size, size, number);
 
       for (j = 0; j < number; ++j)
 	{
@@ -1676,18 +1733,17 @@ pe_print_reloc (abfd, vfile)
 		   _("\treloc %4d offset %4x [%4lx] %s"),
 		   j, off, (long) (off + virtual_address), tbl[t]);
 
-	  /* HIGHADJ takes an argument, but there's no documentation
-	     on what it does, or what it means.  Inferred from
-	     DUMPBIN.  */
+	  /* HIGHADJ takes an argument, - the next record *is* the
+	     low 16 bits of addend.  */ 
 	  if (t == IMAGE_REL_BASED_HIGHADJ)
 	    {
-	       fprintf (file, " (%4x)\n",
+	       fprintf (file, " (%4x)",
 			((unsigned int)
 			 bfd_get_16 (abfd, data + i + 8 + j * 2 + 2)));
 	       j++;
 	    }
-	  else
-	      fprintf (file, "\n");
+	  
+	  fprintf (file, "\n");
 	}
       i += size;
     }
@@ -1708,6 +1764,7 @@ _bfd_pe_print_private_bfd_data_common (abfd, vfile)
   int j;
   pe_data_type *pe = pe_data (abfd);
   struct internal_extra_pe_aouthdr *i = &pe->pe_opthdr;
+  const char *subsystem_name = NULL;
 
   /* The MS dumpbin program reportedly ands with 0xff0f before
      printing the characteristics field.  Not sure why.  No reason to
@@ -1741,12 +1798,44 @@ _bfd_pe_print_private_bfd_data_common (abfd, vfile)
   fprintf (file,"MinorImageVersion\t%d\n", i->MinorImageVersion);
   fprintf (file,"MajorSubsystemVersion\t%d\n", i->MajorSubsystemVersion);
   fprintf (file,"MinorSubsystemVersion\t%d\n", i->MinorSubsystemVersion);
-  fprintf (file,"Reserved1\t\t%08lx\n", i->Reserved1);
+  fprintf (file,"Win32Version\t\t%08lx\n", i->Reserved1);
   fprintf (file,"SizeOfImage\t\t%08lx\n", i->SizeOfImage);
   fprintf (file,"SizeOfHeaders\t\t%08lx\n", i->SizeOfHeaders);
   fprintf (file,"CheckSum\t\t%08lx\n", i->CheckSum);
-  fprintf (file,"Subsystem\t\t%08x\n", i->Subsystem);
-  fprintf (file,"DllCharacteristics\t%08x\n", i->DllCharacteristics);
+  switch (i->Subsystem)
+    {
+    case IMAGE_SUBSYSTEM_UNKNOWN:
+      subsystem_name = "unspecified";
+      break;
+    case IMAGE_SUBSYSTEM_NATIVE:
+      subsystem_name = "NT native";
+      break;
+    case IMAGE_SUBSYSTEM_WINDOWS_GUI:
+      subsystem_name = "Windows GUI";
+      break;
+    case IMAGE_SUBSYSTEM_WINDOWS_CUI:
+      subsystem_name = "Windows CUI";
+      break;
+    case IMAGE_SUBSYSTEM_POSIX_CUI:
+      subsystem_name = "POSIX CUI";
+      break;
+    case IMAGE_SUBSYSTEM_WINDOWS_CE_GUI:
+      subsystem_name = "Wince CUI";
+      break;
+    case IMAGE_SUBSYSTEM_EFI_APPLICATION:
+      subsystem_name = "EFI application";
+      break;
+    case IMAGE_SUBSYSTEM_EFI_BOOT_SERVICE_DRIVER:
+      subsystem_name = "EFI boot service driver";
+      break;
+    case IMAGE_SUBSYSTEM_EFI_RUNTIME_DRIVER:
+      subsystem_name = "EFI runtime driver"; 
+      break;
+    }
+  fprintf (file,"Subsystem\t\t%08x", i->Subsystem);
+  if (subsystem_name)
+    fprintf (file, "\t(%s)", subsystem_name);
+  fprintf (file,"\nDllCharacteristics\t%08x\n", i->DllCharacteristics);
   fprintf (file,"SizeOfStackReserve\t");
   fprintf_vma (file, i->SizeOfStackReserve);
   fprintf (file,"\nSizeOfStackCommit\t");
