@@ -1,5 +1,5 @@
 /* tc-d30v.c -- Assembler code for the Mitsubishi D30V
-   Copyright (C) 1997, 1998, 1999 Free Software Foundation.
+   Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -241,15 +241,19 @@ check_range (num, bits, flags)
      int flags;
 {
   long min, max;
-  int retval=0;
 
-  /* don't bother checking 32-bit values */
-  if (bits == 32 && sizeof(unsigned long) * CHAR_BIT == 32)
-    return 0;
+  /* Don't bother checking 32-bit values.  */
+  if (bits == 32)
+    {
+      if (sizeof(unsigned long) * CHAR_BIT == 32)
+        return 0;
 
-  /* Sign extend signed values to unsigned long */
-  if ((flags & OPERAND_SIGNED) && (num & ((unsigned long)1 << (bits - 1))))
-    num |= ((long)-1 << (bits - 1));
+      /* We don't record signed or unsigned for 32-bit quantities.
+	 Allow either.  */
+      min = -((unsigned long)1 << (bits - 1));
+      max = ((unsigned long)1 << bits) - 1;
+      return (long)num < min || (long)num > max;
+    }
 
   if (flags & OPERAND_SHIFT)
     {
@@ -258,7 +262,7 @@ check_range (num, bits, flags)
       if (flags & OPERAND_SIGNED)
 	num = (unsigned long) ( (long) num >= 0) 
 		? ( ((long) num) >> 3 )
-		: ( (num >> 3) | ((unsigned long)-1 << (32 - 3)) );
+		: ( (num >> 3) | ~(~(unsigned long)0 >> 3) );
       else
 	num >>= 3;
     }
@@ -267,18 +271,14 @@ check_range (num, bits, flags)
     {
       max = ((unsigned long)1 << (bits - 1)) - 1; 
       min = - ((unsigned long)1 << (bits - 1));
-      if (((long)num > max) || ((long)num < min))
-	retval = 1;
+      return (long)num > max || (long)num < min;
     }
   else
     {
       max = ((unsigned long)1 << bits) - 1;
       min = 0;
-      if ((num > max) || (num < min))
-	retval = 1;
+      return num > max || num < min;
     }
-  
-  return retval;
 }
 
 
@@ -582,7 +582,7 @@ build_insn (opcode, opers)
      expressionS *opers;
 {
   int i, length, bits, shift, flags;
-  unsigned int number, id=0;
+  unsigned long number, id=0;
   long long insn;
   struct d30v_opcode *op = opcode->op;
   struct d30v_format *form = opcode->form;
@@ -657,10 +657,10 @@ build_insn (opcode, opers)
       if (bits == 32)
 	{
 	  /* it's a LONG instruction */
-	  insn |= (number >> 26);	/* top 6 bits */
+	  insn |= ((number & 0xffffffff) >> 26);	/* top 6 bits */
 	  insn <<= 32;			/* shift the first word over */
-	  insn |= ((number & 0x03FC0000) << 2);  /* next 8 bits */ 
-	  insn |= number & 0x0003FFFF;		/* bottom 18 bits */
+	  insn |= ((number & 0x03FC0000) << 2);		/* next 8 bits */ 
+	  insn |= number & 0x0003FFFF;			/* bottom 18 bits */
 	}
       else
 	insn |= number << shift;
@@ -814,8 +814,7 @@ write_2_short (opcode1, insn1, opcode2, insn2, exec_type, fx)
 	    }
 	}
       else if ((opcode1->op->flags_used & (FLAG_JMP | FLAG_JSR)
-		&& ((opcode1->op->flags_used & FLAG_DELAY) == 0)
-		&& ((opcode1->ecc == ECC_AL) || ! Optimizing))
+		&& ((opcode1->op->flags_used & FLAG_DELAY) == 0))
 	       || opcode1->op->flags_used & FLAG_RP)
 	{
 	  /* We must emit (non-delayed) branch type instructions
@@ -1146,10 +1145,10 @@ parallel_ok (op1, insn1, op2, insn2, exec_type)
   flags_used1 = op1->op->flags_used;
   flags_used2 = op2->op->flags_used;
 
-  /* ST2W/ST4HB combined with ADDppp/SUBppp is illegal.  */
-  if (((flags_set1 & (FLAG_MEM | FLAG_2WORD)) == (FLAG_MEM | FLAG_2WORD)
+  /* Check for illegal combinations with ADDppp/SUBppp.  */
+  if (((flags_set1 & FLAG_NOT_WITH_ADDSUBppp) != 0
        && (flags_used2 & FLAG_ADDSUBppp) != 0)
-      || ((flags_set2 & (FLAG_MEM | FLAG_2WORD)) == (FLAG_MEM | FLAG_2WORD)
+      || ((flags_set2 & FLAG_NOT_WITH_ADDSUBppp) != 0
 	  && (flags_used1 & FLAG_ADDSUBppp) != 0))
     return 0;
 
