@@ -23,6 +23,7 @@
 #include "as.h"
 #include "subsegs.h"     
 #include "opcode/mn10300.h"
+#include "dwarf2dbg.h"
 
 /* Structure to hold information about predefined registers.  */
 struct reg_name
@@ -31,6 +32,8 @@ struct reg_name
   int value;
 };
 
+struct dwarf2_line_info debug_line;
+
 /* Generic assembler global variables which must be defined by all targets. */
 
 /* Characters which always start a comment. */
@@ -118,6 +121,8 @@ size_t md_longopts_size = sizeof(md_longopts);
 /* The target specific pseudo-ops which we support.  */
 const pseudo_typeS md_pseudo_table[] =
 {
+  { "file", dwarf2_directive_file },
+  { "loc", dwarf2_directive_loc },
   { "am30",	set_arch_mach,		AM30 },
   { "am33",	set_arch_mach,		AM33 },
   { "mn10300",	set_arch_mach,		MN103 },
@@ -125,6 +130,7 @@ const pseudo_typeS md_pseudo_table[] =
 };
 
 #define HAVE_AM33 (current_machine == AM33)
+#define HAVE_AM30 (current_machine == AM30)
 
 /* Opcode hash table.  */
 static struct hash_control *mn10300_hash;
@@ -944,7 +950,8 @@ md_assemble (str)
       /* If the instruction is not available on the current machine
 	 then it can not possibly match.  */
       if (opcode->machine
-	  && !(opcode->machine == AM33 && HAVE_AM33))
+	  && !(opcode->machine == AM33 && HAVE_AM33)
+	  && !(opcode->machine == AM30 && HAVE_AM30))
 	goto error;
 
       for (op_idx = 1, opindex_ptr = opcode->operands;
@@ -1743,6 +1750,21 @@ keep_going:
 	    }
 	}
     }
+
+  if (debug_type == DEBUG_DWARF2)
+    {
+      bfd_vma addr;
+
+      /* First update the notion of the current source line.  */
+      dwarf2_where (&debug_line);
+
+      /* We want the offset of the start of this instruction within the
+         the current frag.  */
+      addr = frag_now->fr_address + frag_now_fix () - size;
+
+      /* And record the information.  */
+      dwarf2_gen_line_info (addr, &debug_line);
+    }
 }
 
 
@@ -1997,3 +2019,10 @@ set_arch_mach (mach)
 
   current_machine = mach;
 }
+
+void
+mn10300_finalize ()
+{
+  if (debug_type == DEBUG_DWARF2)
+    dwarf2_finish ();
+}
