@@ -1,5 +1,5 @@
 /* This file is tc-sh.h
-   Copyright (C) 1993, 94, 95, 96, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1993, 94, 95, 96, 97, 98, 99, 2000 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -24,6 +24,11 @@
 
 #define TARGET_ARCH bfd_arch_sh
 
+#if ANSI_PROTOTYPES
+struct segment_info_struct;
+struct internal_reloc;
+#endif
+
 /* Whether in little endian mode.  */
 extern int shl;
 
@@ -49,6 +54,12 @@ extern void sh_handle_align PARAMS ((fragS *));
 #define TC_FORCE_RELOCATION(fix) sh_force_relocation (fix)
 extern int sh_force_relocation ();
 
+#ifdef OBJ_ELF
+#define obj_fix_adjustable(fixP) sh_fix_adjustable(fixP)
+struct fix;
+extern boolean sh_fix_adjustable PARAMS ((struct fix *));
+#endif
+
 #define IGNORE_NONSTANDARD_ESCAPES
 
 #define LISTING_HEADER (shl ? "Hitachi Super-H GAS Little Endian" : "Hitachi Super-H GAS Big Endian")
@@ -105,7 +116,9 @@ extern void sh_frob_file PARAMS ((void));
 
 #define TC_RELOC_MANGLE(seg, fix, int, paddr) \
   sh_coff_reloc_mangle ((seg), (fix), (int), (paddr))
-extern void sh_coff_reloc_mangle ();
+extern void sh_coff_reloc_mangle
+  PARAMS ((struct segment_info_struct *, struct fix *,
+	   struct internal_reloc *, unsigned int));
 
 #define tc_coff_symbol_emit_hook(a) ; /* not used */
 
@@ -116,14 +129,20 @@ extern void sh_coff_reloc_mangle ();
 #define TC_COFF_SIZEMACHDEP(frag) tc_coff_sizemachdep(frag)
 extern int tc_coff_sizemachdep PARAMS ((fragS *));
 
+#ifdef BFD_ASSEMBLER
+#define SEG_NAME(SEG) segment_name (SEG)
+#else
+#define SEG_NAME(SEG) obj_segment_name (SEG)
+#endif
+
 /* We align most sections to a 16 byte boundary.  */
-#define SUB_SEGMENT_ALIGN(SEG)					\
-  (strncmp (obj_segment_name (SEG), ".stabstr", 8) == 0		\
-   ? 0								\
-   : ((strncmp (obj_segment_name (SEG), ".stab", 5) == 0	\
-       || strcmp (obj_segment_name (SEG), ".ctors") == 0	\
-       || strcmp (obj_segment_name (SEG), ".dtors") == 0)	\
-      ? 2							\
+#define SUB_SEGMENT_ALIGN(SEG)				\
+  (strncmp (SEG_NAME (SEG), ".stabstr", 8) == 0		\
+   ? 0							\
+   : ((strncmp (SEG_NAME (SEG), ".stab", 5) == 0	\
+       || strcmp (SEG_NAME (SEG), ".ctors") == 0	\
+       || strcmp (SEG_NAME (SEG), ".dtors") == 0)	\
+      ? 2						\
       : (sh_small ? 2 : 4)))
 
 #endif /* OBJ_COFF */
@@ -136,6 +155,9 @@ extern int target_big_endian;
 
 #define TARGET_FORMAT (shl ? "elf32-shl" : "elf32-sh")
 
+#define elf_tc_final_processing sh_elf_final_processing
+extern void sh_elf_final_processing PARAMS ((void));
+
 #endif /* OBJ_ELF */
 
 /* end of tc-sh.h */
