@@ -1,4 +1,4 @@
-SCRIPT_NAME=elfmips
+SCRIPT_NAME=elf
 OUTPUT_FORMAT="elf32-littlemips"
 BIG_OUTPUT_FORMAT="elf32-bigmips"
 LITTLE_OUTPUT_FORMAT="elf32-littlemips"
@@ -7,11 +7,13 @@ DATA_ADDR=0x10000000
 MAXPAGESIZE=0x40000
 NONPAGED_TEXT_START_ADDR=0x0400000
 SHLIB_TEXT_START_ADDR=0x5ffe0000
-OTHER_READONLY_SECTIONS='.reginfo : { *(.reginfo) }'
+TEXT_DYNAMIC=
+INITIAL_READONLY_SECTIONS='.reginfo : { *(.reginfo) }'
+OTHER_TEXT_SECTIONS='*(.mips16.fn.*) *(.mips16.call.*)'
 OTHER_GOT_SYMBOLS='
   _gp = ALIGN(16) + 0x7ff0;
 '
-OTHER_READWRITE_SECTIONS='
+OTHER_GOT_SECTIONS='
   .lit8 : { *(.lit8) }
   .lit4 : { *(.lit4) }
 '
