@@ -45,6 +45,8 @@
 # define N_(String) (String)
 #endif
 
+#include "bin-bugs.h"
+
 /* Look in this environment name for the linker to pretend to be */
 #define EMULATION_ENVIRON "LDEMULATION"
 /* If in there look for the strings: */
@@ -235,8 +237,8 @@ extern void check_nocrossrefs PARAMS ((void));
 extern void ld_abort PARAMS ((const char *, int, const char *))
      ATTRIBUTE_NORETURN;
 
-/* If gcc, we can give a function name, too.  */
-#if !defined (__GNUC__) || __GNUC_MINOR__ <= 5
+/* If gcc >= 2.6, we can give a function name, too.  */
+#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 6)
 #define __PRETTY_FUNCTION__  ((char*) NULL)
 #endif
 
