@@ -1,5 +1,5 @@
 /* ld-emul.h - Linker emulation header file
-   Copyright 1991, 92, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
+   Copyright 1991, 92, 93, 94, 95, 96, 97, 1998, 2000 Free Software Foundation, Inc.
 
    This file is part of GLD, the Gnu Linker.
 
@@ -32,6 +32,7 @@ extern void ldemul_set_output_arch PARAMS ((void));
 extern char *ldemul_choose_target PARAMS ((void));
 extern void ldemul_choose_mode PARAMS ((char *));
 extern void ldemul_list_emulations PARAMS ((FILE *));
+extern void ldemul_list_emulation_options PARAMS ((FILE *));
 extern char *ldemul_get_script PARAMS ((int *isfile));
 extern void ldemul_finish PARAMS ((void));
 extern void ldemul_set_symbols PARAMS ((void));
@@ -41,6 +42,8 @@ extern boolean ldemul_place_orphan
 extern int ldemul_parse_args PARAMS ((int, char **));
 extern boolean ldemul_unrecognized_file
   PARAMS ((struct lang_input_statement_struct *));
+extern boolean ldemul_recognized_file
+  PARAMS ((struct lang_input_statement_struct *));
 extern boolean ldemul_open_dynamic_archive
   PARAMS ((const char *, struct search_dirs *,
 	   struct lang_input_statement_struct *));
@@ -52,6 +55,8 @@ extern void before_allocation_default PARAMS ((void));
 extern void set_output_arch_default PARAMS ((void));
 extern void syslib_default PARAMS ((char*));
 extern void hll_default PARAMS ((char*));
+extern int  ldemul_find_potential_libraries
+  PARAMS ((char *, struct lang_input_statement_struct *));
 
 typedef struct ld_emulation_xfer_struct
 {
@@ -124,6 +129,21 @@ typedef struct ld_emulation_xfer_struct
   boolean (*unrecognized_file)
     PARAMS ((struct lang_input_statement_struct *));
 
+  /* Run to list the command line options which parse_args handles.  */
+  void (* list_options) PARAMS ((FILE *));
+
+  /* Run to specially handle files which *are* recognized as object
+     files or archives.  Return true if the file was handled.  */
+  boolean (*recognized_file)
+    PARAMS ((struct lang_input_statement_struct *));
+
+  /* Called when looking for libraries in a directory specified
+     via a linker command line option or linker script option.
+     Files that match the pattern "lib*.a" have already been scanned.
+     (For VMS files matching ":lib*.a" have also been scanned).  */
+  int (* find_potential_libraries)
+    PARAMS ((char *, struct lang_input_statement_struct *));
+  
 } ld_emulation_xfer_type;
 
 typedef enum 
