@@ -1,5 +1,5 @@
 /* Parse options for the GNU linker.
-   Copyright (C) 1991, 92, 93, 94, 95, 96, 97, 98, 1999
+   Copyright (C) 1991, 92, 93, 94, 95, 96, 97, 98, 99, 2000
    Free Software Foundation, Inc.
 
 This file is part of GLD, the Gnu Linker.
@@ -338,7 +338,7 @@ static const struct ld_option ld_options[] =
       '\0', N_("COUNT"), N_("Split output sections every COUNT relocs"), TWO_DASHES },
   { {"stats", no_argument, NULL, OPTION_STATS},
       '\0', NULL, N_("Print memory usage statistics"), TWO_DASHES },
-  { {"style", no_argument, NULL, OPTION_STYLE},
+  { {"style", required_argument, NULL, OPTION_STYLE},
       '\0', "{gnu,lucid,arm,hp,edg,gnat,compaq}", N_("Set demangling style"), TWO_DASHES },
   { {"task-link", required_argument, NULL, OPTION_TASK_LINK},
       '\0', N_("SYMBOL"), N_("Do task level linking"), TWO_DASHES },
@@ -1071,15 +1071,13 @@ static void
 set_section_start (sect, valstr)
      char *sect, *valstr;
 {
-  char *end;
-  unsigned long val = strtoul (valstr, &end, 16);
+  const char *end;
+  bfd_vma val = bfd_scan_vma (valstr, &end, 16);
   if (*end)
     einfo (_("%P%F: invalid hex number `%s'\n"), valstr);
   lang_section_start (sect, exp_intop (val));
 }
 
-extern void binutils_bug_report PARAMS((FILE *));
-
 /* Print help messages for the options.  */
 
 static void
@@ -1184,7 +1182,7 @@ help ()
   /* xgettext:c-format */
   printf (_("%s: emulation specific options:\n"), program_name);
   ldemul_list_emulation_options (stdout);
+  printf ("\n");
 
-  printf ("\n\n");
-  binutils_bug_report (stdout);
+  printf (_("Report bugs to %s\n"), REPORT_BUGS_TO);
 }
